%-------------------------------------------------
% PROGRAM: EnKF
%  Solves the Lorenz-63 equations using an extended Kalman filter 
%
% AUTHOR:Alison Fowler
%-------------------------------------------------
clear all
close all
% Determine screen size and set up figure positions
ss=get(0,'ScreenSize');
fig_width=0.55*ss(3);
fig_height=0.4*ss(4);
pos_1=[0.02*ss(3),0.5*ss(4),fig_width,fig_height];
pos_2=[0.02*ss(3),0.05*ss(4),fig_width,fig_height];
pos_3=[0.6*ss(3),0.45*ss(4),0.4*ss(3),0.45*ss(4)];
pl=0.6*ss(3);
pb=0.05*ss(4);
pw=0.3*ss(3);
ph=0.35*ss(4);
pos_4=[pl,pb,pw,ph];
pos_5=[0.02*pw,0.8*ph,0.95*pw,0.2*ph];
pos_6=[0.02*pw,0.02*ph,0.95*pw,0.75*ph];

% SETUP

N = 3; %total number of variables
MO = 3; % Number of observations at analysis time
randn('state',0) % random seed
%lorenz model parameters
sigma = 10.;
rho = 28.;
beta = 8./3.; 

%------------------------

% INPUT PARAMETERS

% Inputs 1: initial data
xstr='True value of x at t=0';
ystr='True value of y at t=0';
zstr='True value of z at t=0';
astr={'1.5','-1.5','25.5'}; % suggested input parameters
true_str=inputdlg({xstr,ystr,zstr},'Initial data',1,astr);
truex=str2num(true_str{1});
truey=str2num(true_str{2});
truez=str2num(true_str{3});

% Inputs 2
xstr='Length of assimilation window';
x2str='Length of forecast';
ystr='Time step';
zstr='Frequency of observations (in time steps)';
astr={'3','1','0.01','20'}; % suggested input parameters
temp_str=inputdlg({xstr,x2str,ystr,zstr},'Run information',1,astr);
T=str2num(temp_str{1});
fT=str2num(temp_str{2});
dt=str2num(temp_str{3});
ns=str2num(temp_str{4});
T=T/dt;
fT=fT/dt;


%------------------------

% CALCULATE THE TRUTH
x(1,1) =truex;
x(2,1) =truey ;
x(3,1) =truez ;
%initial truth is the background plus a random error proportional to error
%in the initial conditions
svar=sqrt(1.);
xs(1,1) = truex + svar * randn(1,1);
xs(2,1) = truey + svar * randn(1,1);
xs(3,1) = truez + svar * randn(1,1);

for t=2:T+fT
  random = randn(3,1);
%  force = scov_model * random; %random forcing proportinal to error in the model
  noforce=zeros(N,1);
%  x(:,t) = slorenz_rk4(x(:,t-1),dt, sigma, rho, beta,noforce); 
  x(:,t)=modeuler(dt,x(:,t-1), sigma, rho, beta,noforce);
 % xs(:,t) = slorenz_rk4(xs(:,t-1),dt, sigma, rho, beta, force);
%  xs(:,t)=modeuler(dt,x(:,t-1), sigma, rho, beta,force);
end

xtrue = x;

%plot the truth
h1=figure('Position',pos_1);
clf;

subplot(3,1,1)
hold on
plot([1:T+fT]*dt,xtrue(1,:),'k')
axis([0 (T+fT)*dt -20 20]);
ylabel('X')
title(['EKF'])
legend('truth','location','SouthOutside','orientation','horizontal')
subplot(3,1,2)
hold on
plot([1:T+fT]*dt,xtrue(2,:),'k')
axis([0 (T+fT)*dt -25 25]);
ylabel('Y')
subplot(3,1,3)
hold on
plot([1:T+fT]*dt,xtrue(3,:),'k')
axis([0 (T+fT)*dt 5 45]);
xlabel('time')
ylabel('Z')
%-----------------------

% CALCULATE OBSERVATIONS

NM = floor(T/ns);  %number of measurement times
y = zeros(MO,NM);
Msqrt_cov = zeros(MO,MO);

% Inputs 3
xstr='Observation error variance';
astr={'1.'}; % suggested input parameters
temp_str=inputdlg({xstr},'',1,astr);
varobs=str2num(temp_str{1});
sobs=sqrt(varobs);
R_structure=[1,0,0.;0.,1,0.;0.,0.,1];
  R=varobs*R_structure;
  [V,L] = eig(R);
  Msqrt_cov=V*sqrt(L);
  M_cov = Msqrt_cov*Msqrt_cov';

for t = 1:NM     %number of  measurement times
   y(:,t) = xtrue(:,t*ns) + Msqrt_cov* randn(3,1); %obs=truth+random error proportional to ob error std dev
end
inv_cov = pinv(M_cov); 

% plot observations
subplot(3,1,1)
plot((ns:ns:T)*dt,y(1,:),'r*')
legend('truth','observations','location','SouthOutside','orientation','horizontal')
subplot(3,1,2)
plot((ns:ns:T)*dt,y(2,:),'r*')
subplot(3,1,3)
plot((ns:ns:T)*dt,y(3,:),'r*')
%-----------------------

% CALCULATE PRIOR GUESS AT INITIAL TIME
xb = zeros(1,N);
Bsqrt_cov = zeros(N,N);

% Inputs 3
xstr='Background error variance';
astr={'1.'}; % suggested input parameters
temp_str=inputdlg({xstr},'',1,astr);
varb=str2num(temp_str{1});
sb=sqrt(varb);
B_structure=[1,0,0.;0.,1,0.;0.,0.,1];
B=varb*B_structure;
[V,L] = eig(B);
Bsqrt_cov=V*sqrt(L);

xb=xtrue(:,1)+Bsqrt_cov* randn(3,1); 

%---------------------------------

%-----------------------
% BEGIN EnKF
%Q=eye(3);%
%These are from Evensen, 1997.
  Q(1,1)=0.1491;
  Q(1,2)=0.1505;
  Q(1,3)=0.0007;
  Q(2,1)=0.1505;
  Q(2,2)=0.9048;
  Q(2,3)=0.0014;
  Q(3,1)=0.0007;
  Q(3,2)=0.0014;
  Q(3,3)=0.9180;
 % Q=eye(3).*Q;
 Q
xf(:,1)=xb;
x_f(:,1)=xb;
Pf(:,:,1)=B;
 mcn = 0; %a counting index of analyses
 for t = 2:T 
     xf(:,t)=modeuler(dt,xf(:,t-1), sigma, rho, beta,noforce);
     [M,M_tr]=tl_adj(dt,sigma,beta,rho,xf(:,t));
     Pf(:,:,t)=M*Pf(:,:,t-1)*M_tr+Q;
     x_f(:,t)=M*x_f(:,t-1);
     if (mod(t-1,ns) == 0) && (t > 1) 
                                   % this is only true at measurement times
       
        mcn = mcn + 1  ;  
        
        xa(:,mcn)=xf(:,t)+Pf(:,:,t)*pinv(R+Pf(:,:,t))*(y(:,mcn)-xf(:,t));
        Pa(:,:,mcn)=Pf(:,:,t)-Pf(:,:,t)*pinv(R+Pf(:,:,t))*Pf(:,:,t);
     xf(:,t)=xa(:,mcn);
     Pf(:,:,t)=Pa(:,:,mcn);
     x_f(:,t)=xa(:,mcn);
     end
 end
%% 
 %plot results
subplot(3,1,1)
plot([1:T]*dt,xf(1,:),'b')
%plot([1:T]*dt,x_f(1,:),'c')
box on
legend('truth','observations','analysis',...
    'location','SouthOutside','orientation','horizontal')
subplot(3,1,2)
plot([1:T]*dt,xf(2,:),'b')
%plot([1:T]*dt,x_f(2,:),'c')
box on
subplot(3,1,3)
plot([1:T]*dt,xf(3,:),'b')
%plot([1:T]*dt,x_f(3,:),'c')
box on
 
       
%%
% plot forecast
%figure(1)
for t=T+1:T+fT
%  xem(:,t)= slorenz_rk4(xem(:,t-1),dt, sigma, rho, beta,noforce);
       xf(:,t)=modeuler(dt,xf(:,t-1), sigma, rho, beta,noforce);
end
subplot(3,1,1)
plot([1:T+fT]*dt,xf(1,:),'b-');
plot(T*[1 1]*dt,ylim,'b:')
box on
 subplot(3,1,2)
plot([1:T+fT]*dt,xf(2,:),'b-');
plot(T*[1 1]*dt,ylim,'b:')
box on
 subplot(3,1,3)
plot([1:T+fT]*dt,xf(3,:),'b-');
plot(T*[1 1]*dt,ylim,'b:')
box on


%plot root mean square error and ensemble variance
rmse=(xf-xtrue).*(xf-xtrue);
%%
h1=figure('Position',pos_2);
clf;
subplot(3,1,1)
hold on
plot([1:T]*dt,rmse(1,1:T),'k');
plot([1:T]*dt,squeeze(Pf(1,1,:)),'r');
box on
legend('(x_a-x_{truth})^2','analysis error variance',...
    'location','EastOutside','orientation','vertical')
subplot(3,1,2)
hold on
plot([1:T]*dt,rmse(2,1:T),'k');
plot([1:T]*dt,squeeze(Pf(2,2,:)),'r');
box on
legend('(y_a-y_{truth})^2','analysis error variance',...
    'location','EastOutside','orientation','vertical')
subplot(3,1,3)
hold on
plot([1:T]*dt,rmse(3,1:T),'k');
plot([1:T]*dt,squeeze(Pf(3,3,:)),'r');
box on
legend('(z_a-z_{truth})^2','analysis error variance',...
    'location','EastOutside','orientation','vertical')
xlabel('time')
        
 %-----------------------------------------------------------------------
% 9. Write out options chosen
%---------------------------------------------------------------------
h4=figure('Position',pos_4);
clf;
text1={'List of options chosen for the Extended Kalman filter'};
text2={['True (x,y,z) at t=0:   (' true_str{1} ',' true_str{2} ',' ...
 true_str{3}  ')']};
text4={['Length of assimilation window: ' num2str(T)]};
text5={['Length of subsequent forecast: ' num2str(fT) ]};
text6={['Time step: ' num2str(dt)]};
text7={['Frequency of observations = ' num2str(ns) ]};
text11={['Observations have random noise with variance ' num2str(varobs)]};
text12={['Background noise = ',num2str(varb)]};

str1=[text2;text4;text5;text6;text7;text11;text12];
uicontrol('Style','text','Position',pos_5,'String',text1,...
 'FontSize',14,'FontWeight','bold')
uicontrol('Style','text','Position',pos_6,'String',str1,...
 'FontSize',12,'HorizontalAlignment','left')
 
 
 
 




