%-------------------------------------------------
% PROGRAM: EnKF
%  Solves the Lorenz-63 equations using an ensemble Kalman filter with  perturbed observations
%
% AUTHOR:Alison Fowler
%-------------------------------------------------
clear all
close all
% Determine screen size and set up figure positions
ss=get(0,'ScreenSize');
fig_width=0.55*ss(3);
fig_height=0.4*ss(4);
pos_1=[0.02*ss(3),0.5*ss(4),fig_width,fig_height];
pos_2=[0.02*ss(3),0.05*ss(4),fig_width,fig_height];
pos_3=[0.6*ss(3),0.45*ss(4),0.4*ss(3),0.45*ss(4)];
pl=0.6*ss(3);
pb=0.05*ss(4);
pw=0.3*ss(3);
ph=0.35*ss(4);
pos_4=[pl,pb,pw,ph];
pos_5=[0.02*pw,0.8*ph,0.95*pw,0.2*ph];
pos_6=[0.02*pw,0.02*ph,0.95*pw,0.75*ph];

% SETUP

N = 3; %total number of variables
MO = 3; % Number of observations at analysis time
randn('state',0) % random seed
%lorenz model parameters
sigma = 10.;
rho = 28.;
beta = 8./3.; 

%------------------------

% INPUT PARAMETERS

% Inputs 1: initial data
xstr='True value of x at t=0';
ystr='True value of y at t=0';
zstr='True value of z at t=0';
astr={'1.5','-1.5','25.5'}; % suggested input parameters
true_str=inputdlg({xstr,ystr,zstr},'Initial data',1,astr);
truex=str2num(true_str{1});
truey=str2num(true_str{2});
truez=str2num(true_str{3});

% Inputs 2
xstr='Length of assimilation window';
x2str='Length of forecast';
ystr='Time step';
zstr='Frequency of observations (in time steps)';
z2str='Number of ensemble members';
astr={'3','1','0.01','20','50'}; % suggested input parameters
temp_str=inputdlg({xstr,x2str,ystr,zstr,z2str},'Run information',1,astr);
T=str2num(temp_str{1});
fT=str2num(temp_str{2});
dt=str2num(temp_str{3});
ns=str2num(temp_str{4});
MM=str2num(temp_str{5});
T=T/dt;
fT=fT/dt;

%Q=eye(3);%
%These are from Evensen, 1997.
  Q(1,1)=0.1491;
  Q(1,2)=0.1505;
  Q(1,3)=0.0007;
  Q(2,1)=0.1505;
  Q(2,2)=0.9048;
  Q(2,3)=0.0014;
  Q(3,1)=0.0007;
  Q(3,2)=0.0014;
  Q(3,3)=0.9180;
 %Q=eye(3).*Q;
 Q
 cov_model = dt * Q;
 scov_model = sqrt(cov_model); % model error covariance model
%------------------------

% CALCULATE THE TRUTH
x(1,1) =truex;
x(2,1) =truey ;
x(3,1) =truez ;
%initial truth is the background plus a random error proportional to error
%in the initial conditions
svar=sqrt(1.);
xs(1,1) = truex + svar * randn(1,1);
xs(2,1) = truey + svar * randn(1,1);
xs(3,1) = truez + svar * randn(1,1);

for t=2:T+fT
  random = randn(3,1);
  force = scov_model * random; %random forcing proportinal to error in the model
  noforce=zeros(N,1);
%  x(:,t) = slorenz_rk4(x(:,t-1),dt, sigma, rho, beta,noforce); 
  x(:,t)=modeuler(dt,x(:,t-1), sigma, rho, beta,noforce);
 % xs(:,t) = slorenz_rk4(xs(:,t-1),dt, sigma, rho, beta, force);
  xs(:,t)=modeuler(dt,x(:,t-1), sigma, rho, beta,force);
end

xtrue = x;

%plot the truth
h1=figure('Position',pos_1);
clf;

subplot(3,1,1)
hold on
plot([1:T+fT]*dt,xtrue(1,:),'k')
axis([0 (T+fT)*dt -20 20]);
ylabel('X')
title(['EnKF:',num2str(MM),' ensemble members'])
legend('truth','location','SouthOutside','orientation','horizontal')
subplot(3,1,2)
hold on
plot([1:T+fT]*dt,xtrue(2,:),'k')
axis([0 (T+fT)*dt -25 25]);
ylabel('Y')
subplot(3,1,3)
hold on
plot([1:T+fT]*dt,xtrue(3,:),'k')
axis([0 (T+fT)*dt 5 45]);
xlabel('time')
ylabel('Z')
%-----------------------

% CALCULATE OBSERVATIONS

NM = floor(T/ns);  %number of measurement times
y = zeros(MO,NM);
Msqrt_cov = zeros(MO,MO);

% Inputs 3
xstr='Observation error variance';
astr={'1.'}; % suggested input parameters
temp_str=inputdlg({xstr},'',1,astr);
varobs=str2num(temp_str{1});
sobs=sqrt(varobs);
R_structure=[1,0,0.;0.,1,0.;0.,0.,1];
  R=varobs*R_structure;
  [V,L] = eig(R);
  Msqrt_cov=V*sqrt(L);
  M_cov = Msqrt_cov*Msqrt_cov';

for t = 1:NM     %number of  measurement times
    for i=1:MO   %number of observations at measurement time
       y(i,t) = xtrue(i,t*ns) + Msqrt_cov(i,i) * randn; %obs=truth+random error proportional to ob error std dev
    end
end
inv_cov = pinv(M_cov); 

% plot observations
subplot(3,1,1)
plot((ns:ns:T)*dt,y(1,:),'r*')
legend('truth','observations','location','SouthOutside','orientation','horizontal')
subplot(3,1,2)
plot((ns:ns:T)*dt,y(2,:),'r*')
subplot(3,1,3)
plot((ns:ns:T)*dt,y(3,:),'r*')
%-----------------------

% BEGIN EnKF

 randn('state',0) % random seed
 xp = zeros(N,MM); %particle values
 yp = zeros(MO,MM); % 'observation' values for each particle
 xem = zeros(N,T);  %mean particle value
 %ym = zeros(MO,1);
 henkf = zeros(MO,MM,2,NM);
 xdiff = zeros(T,1);
 
% Inputs 4
xstr='Background error variance';
astr={'1.'}; % suggested input parameters
temp_str=inputdlg({xstr},'',1,astr);
varb=str2num(temp_str{1});

 mcn = 0; %a counting index of analyses
 for t = 1:T 
   for m = 1:MM   %loop over each enseble member
       if (t == 1)  %initialise ensemble memebers
  
  B_structure=[1,0.5,0.25;0.5,1,0.5;0.25,0.5,1];
  B=varb*B_structure;
  [V,L] = eig(B);
  B_sqrt=V*sqrt(L);
  B_eeror=B_sqrt*randn(3,1);
       xp(1,m) = truex + B_eeror(1);
       xp(2,m) = truey + B_eeror(2);
       xp(3,m) = truez + B_eeror(3);
     else
       random = randn(3,1);
       force = scov_model * random;
 %      xp(:,m) = slorenz_rk4(xp(:,m),dt, sigma, rho, beta, force);
       xp(:,m)=modeuler(dt,xp(:,m), sigma, rho, beta,force);
                                    %propogate each member forward to time T
     end

     if (mod(t-1,ns) == 0) && (t > 1) 
                                   % this is only true at measurement times
       if (m == 1)  
          mcn = mcn + 1  ;  
       end
       % measure member
       for i=1:MO
         yp(i,m) = xp(i,m);  %model version of ob for each ensemble member 
                             % at time of observation- these are direct measurements
       end
    end %mod loop
     
   end %m loop (ensemble member)
   
   % Determine ensemble mean
   for i=1:N  % loop over each variable
       xem(i,t) = 0;
       for m=1:MM  % loop over ensemble members
           xem(i,t) = xem(i,t) + xp(i,m)/MM; %ensemble mean
       end
%       xem(i,t) = xem(i,t)/MM;
   end
   
   if (mod(t-1,ns) == 0) % this is only true at measurement times       
       if (t > 1)
           
       % Some ensemble statistics
       for i=1:MO % loop over number of obs at analysis time (1)
           for m=1:MM %loop over number of ensembles
               henkf(i,m,1,(t-1)/ns) = xp(i,m); % matrix of ensemble 
                                                %member value for each ob 
                                                %variable at ob times
           end   
       end
       
       % Determine mean observation from model ensemble
       for i=1:MO %loop over number of obs at analysis time
           ym(i) = 0;
           for m=1:MM
               ym(i) = ym(i) + yp(i,m);
           end
           ym(i) = ym(i)/MM; %mean value of ensemble version of ob
       end
       
       % calculate kalman gain       
       PH = zeros(N,MO);
       HPH = zeros(MO,MO);
       for m = 1:MM %loop over ensemble members
           PH =  PH + (xp(:,m) - xem(:,t))*(yp(:,m) - ym')'/(MM-1.);
           HPH = HPH + (yp(:,m) - ym')*(yp(:,m) - ym')'/(MM-1.);
       end
       
       K = PH*pinv(HPH + M_cov);

       %measurement assimilation for EnKF (with perturbed observations)
      
       for m = 1:MM
           % at time of ob and analysis
          xp(:,m) = xp(:,m) + K*(y(:,mcn) - yp(:,m) + Msqrt_cov*randn(MO,1));
                                                        
       end
       for i=1:N
           xem(i,t) = 0;
           for m=1:MM
               xem(i,t) = xem(i,t) + xp(i,m);
           end
           xem(i,t) = xem(i,t)/MM; %calc ens mean of analysis
       end

       % Some ensemble statistics
       for i=1:MO
           for m=1:MM
               henkf(i,m,2,(t-1)/ns) = xp(i,m);% matrix of analysis ensemble 
                                                %member value for each ob 
                                                %variable at ob times
           end
       end
       
       end % t>1
         
    end %mod loop (ob time)
    
    for i=1:N
        for j=1:MM
            xens_all(i,j,t)=xp(i,j);
        end
    end
  end %  T loop
  
      
%%
%figure(1) 
g=[0.6 0.6 0.6]; 
 % plot ensemble
 subplot(3,1,1)
        plot([1:T]*dt,squeeze(xens_all(1,1,:)),'color',g);
        plot([1:T]*dt,xem(1,:),'b-','linewidth',2);
legend('truth','observations','ensemble member','ensemble mean',...
    'location','SouthOutside','orientation','horizontal')
        for m=2:MM
            plot([1:T]*dt,squeeze(xens_all(1,m,:)),'color',g);
        end
plot((ns:ns:T)*dt,y(1,:),'r*')
box on
 subplot(3,1,2)
        for m=1:MM
            plot([1:T]*dt,squeeze(xens_all(2,m,:)),'color',g);
        end
plot((ns:ns:T)*dt,y(2,:),'r*')
box on
 subplot(3,1,3)
        for m=1:MM
            plot([1:T]*dt,squeeze(xens_all(3,m,:)),'color',g);
        end
plot((ns:ns:T)*dt,y(3,:),'r*')
box on
%%
% plot forecast
%figure(1)
for t=T+1:T+fT
%  xem(:,t)= slorenz_rk4(xem(:,t-1),dt, sigma, rho, beta,noforce);
       xem(:,t)=modeuler(dt,xem(:,t-1), sigma, rho, beta,noforce);
end
subplot(3,1,1)
plot([1:T+fT]*dt,xem(1,:),'b-','linewidth',2);
plot([1:T+fT]*dt,xtrue(1,:),'k')
plot(T*[1 1]*dt,ylim,'b:')
 subplot(3,1,2)
plot([1:T+fT]*dt,xem(2,:),'b-','linewidth',2);
plot([1:T+fT]*dt,xtrue(2,:),'k')
plot(T*[1 1]*dt,ylim,'b:')
 subplot(3,1,3)
plot([1:T+fT]*dt,xem(3,:),'b-','linewidth',2);
plot([1:T+fT]*dt,xtrue(3,:),'k')
plot(T*[1 1]*dt,ylim,'b:')


%plot root mean square error and ensemble variance
rmse=(xem-xtrue).*(xem-xtrue);
Pa = zeros(N,N,T);
 for t = 1:T
       for m = 1:MM %loop over ensemble members
           Pa(:,:,t) =  Pa(:,:,t) +...
 (xens_all(:,m,t) - xem(:,t))*(xens_all(:,m,t) - xem(:,t))'/(MM-1.);
       end
end
%%
h1=figure('Position',pos_2);
clf;
subplot(3,1,1)
hold on
plot([1:T]*dt,rmse(1,1:T),'k');
plot([1:T]*dt,squeeze(Pa(1,1,:)),'r');
box on
legend('(x_a-x_{truth})^2','Ensemble spread',...
    'location','EastOutside','orientation','vertical')
subplot(3,1,2)
hold on
plot([1:T]*dt,rmse(2,1:T),'k');
plot([1:T]*dt,squeeze(Pa(2,2,:)),'r');
box on
legend('(y_a-y_{truth})^2','Ensemble spread',...
    'location','EastOutside','orientation','vertical')
subplot(3,1,3)
hold on
plot([1:T]*dt,rmse(3,1:T),'k');
plot([1:T]*dt,squeeze(Pa(3,3,:)),'r');
box on
legend('(z_a-z_{truth})^2','Ensemble spread',...
    'location','EastOutside','orientation','vertical')
xlabel('time')
        
 %-----------------------------------------------------------------------
% 9. Write out options chosen
%---------------------------------------------------------------------
h4=figure('Position',pos_4);
clf;
text1={'List of options chosen for the Ensemble Kalman filter'};
text2={['True (x,y,z) at t=0:   (' true_str{1} ',' true_str{2} ',' ...
 true_str{3}  ')']};
text4={['Length of assimilation window: ' num2str(T*dt)]};
text5={['Length of subsequent forecast: ' num2str(fT*dt) ]};
text6={['Time step: ' num2str(dt)]};
text7={['Frequency of observations = ' num2str(ns) ]};
text8={['Ensemble size = ' num2str(MM) ]};
text11={['Observations have random noise with variance ' num2str(varobs)]};
text12={['Background noise = ',num2str(varb)]};

str1=[text2;text4;text5;text6;text7;text8;text11;text12];
uicontrol('Style','text','Position',pos_5,'String',text1,...
 'FontSize',14,'FontWeight','bold')
uicontrol('Style','text','Position',pos_6,'String',str1,...
 'FontSize',12,'HorizontalAlignment','left')
 
 
 
